const light = require('./kernel');
const lighting_str = `
     _  _         _      _    _               
    | |(_)       | |    | |  (_)              
    | | _   __ _ | |__  | |_  _  _ __    __ _ 
    | || | / _\` || '_ \\ | __|| || '_ \\  / _\` |
    | || || (_| || | | || |_ | || | | || (_| |
    |_||_| \\__, ||_| |_| \\__||_||_| |_| \\__, |
            __/ |                        __/ |
           |___/                        |___/      CLI ${require("./package.json").version.green}

`;

function version() {
    console.log();
    console.log(lighting_str);
    // console.log();
    // light.util.readFileSync(`${__dirname}/readme.markdown`).toString().trim().split("###").slice(0,3).forEach(function (line) {
    // 	if(line.indexOf("更新说明")!==0){
    //         console.log(line.trim())
    // }
    // })
}

function help() {
    console.log(`
    ${lighting_str}
    
    Usage: 

    $ ${'light <command>'.green} ${'[options]'.cyan}  ${'[-h]'.cyan}

        ${'create'.green}...........................生成工程的基本目录结构和模板文件
        ${'gen'.green}..............................代码生成
        ${'install'.green}..........................安装项目依赖和插件       
        ${'release'.green}..........................代码的集成打包
        ${'server'.green}...........................内置HTTP服务器
        ${'plugin'.green}...........................插件的安装卸载更新
        ${'login'.green}............................登录账户链接light平台的账户系统以进行应用管理

    Project Commands:
        ${require("fs").existsSync(`${process.cwd()}/package.json`) || require("fs").existsSync(`${process.cwd()}/project.json`) ? `
        ${'light gen'.green}........................在工程的html文件中配置完视图后生成视图的骨架
        ${'light release -w -b'.green}..............开发过程中监视文件变更并打开浏览器
        ${'light release -u -c -s -p'.green}........开发完成后打包输出生成配置包
        ${'light server -r . -p 3000'.green}........使用内置的server提供http服务
        `: `
        You are not in a project directory`}
	`);
}
function main(argv) {

    var cmdArg = argv[2];

    if (!cmdArg) {
        cmdArg = '-h';
    }

    if (cmdArg === '-v' || cmdArg === '--version') {
        version();
    } else if (cmdArg === '-h' || cmdArg === '--help') {
        help();
    } else {
        light.commander.parse(argv);
    }
}

module.exports.run = main;

if (!String.prototype.padEnd) {
    String.prototype.padEnd = function padEnd(targetLength, padString) {
        targetLength = targetLength >> 0; //floor if number or convert non-number to 0;
        padString = String((typeof padString !== 'undefined' ? padString : ' '));
        if (this.length > targetLength) {
            return String(this);
        }
        else {
            targetLength = targetLength - this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
            }
            return String(this) + padString.slice(0, targetLength);
        }
    };
}

function checkUpdate() {
    let checkResult = "";

    if (process.platform === "win32") {
        let rl = require("readline").createInterface({
            input: process.stdin,
            output: process.stdout
        });

        rl.on("SIGINT", function () {
            process.emit("SIGINT");
        });
    }

    process.on('SIGINT', function () {
        process.exit();
    });

    process.on('exit', function () {
        if (checkResult) {
            let latest = JSON.parse(checkResult)['dist-tags'].latest;
            let update_str = `
                _____________________________________________________________________________
               |                                                                             |
               |                         lighting update check success                       |
               |                                                                             |
               |     current_version:${current.green}  latest_version:${latest.green}${" ".padEnd(78 - 39 - current.length - latest.length, " ")}|
               |                                                                             |
               |                     please upgrade to the latest version                    |
               |_____________________________________________________________________________|
                
                
            `;
            if (latest !== current) {
                console.log(update_str);
            }
        }
    });


    let request = require('request');
    let current = require("./package.json").version;
    request.get({
        url: 'https://registry.npm.taobao.org/lighting',
        timeout: 3000
    }, function (err, res, body) {
        if (!err && res.statusCode === 200) {
            checkResult = body;
        }
    })

}

//78858 自动检查更新的功能不能每次都触发,太浪费,设置为1/5的触发概率
if (Math.random() < 0.2) {
checkUpdate();
}

/**
 * 以API的方式执行编译任务
 * @param cmd 指令
 * @param options 参数
 */
module.exports.exec = function (cmd, options) {
    return require(`./service/${cmd}`).do(null, options)
};

/**
 * 配置lighting的命令工具，主要是：
 * 1. 源代码目录
 * 2. 目标代码目录
 * 3. 插件的查找目录等
 */
module.exports.config = function (config) {
    light.util.extend(light.config, config);
    return module.exports;
};