---

title: light-api使用文档
author: 童永亮
tags: node
version: 0.0.1
date: '2017-09-11'
---

light-api是一个可以快速开发RESTful API的富插件的node服务框架。本文档适合有node基础的人员阅读，测试需要安装node环境。


> ##### 安装

```
npm install light-api --save

```

> ##### 新手上路

1. 初始化工程

新建文件夹 my-new-demo 进入my-new-demo文件中,打开cmd 运行命令
```
npm init

```
命令运行结束后，目录会多出package.json文件,目录结构
```
|-package.json
```

2. 安装 light-api，cmd下运行命令
```
npm install light-api --save
```
命令结束后，目录会变成
```
|-package.json
|-node_modules
```

3. 在my-new-demo目录下创建文件api、config、plugin 文件夹，创建app.js文件，目录结构如下：
```
|-api
|-config
|-node_modules
|-plugin
|-app.js
|-package.json

```
4. 配置工程

在config目录中新建文件config.js。编写代码:
```
module.exports = {
    getConfig:function(env){
        //引入server配置
        let svr = require(`./${env}/server.json`);
        return{
            "server":svr,
            "plugins":{
               
            }
        
        }
    }
}

```
在config目录中新建目录test，并在test目录下新建server.json用来配置server属性，server.json内容为：

```
{
    "port":9000,
    "documentation":true
}
```
此时的目录结构为
```
|-api
|-config
    |-config.js
    |-test
        |-server.json
|-node_modules
|-plugin
|-app.js
|-package.json

```

5. 编写启动脚本app.js。app.js的内容为：

```
const app = require("light-api");
//先初始化 ，然后启动App
app.init().start(function(){
   console.log('start...');
});

```

6. 启动工程

这时整个工程已经配置完成，可以启动sever ，在工程的根目录下运行启动命令

```
node app.js test

// 命令中的test 指的是加载config下test目录的配置文件。注意。配置文件必须放置在config目录中。

```
此时会输出如下结果，说明server已经正常启动
```
$ node app.js  test

==============================WELCOME================================
  Welcome to LIGHT-API !!!
  Start env: test
  Server runing at: http://localhost:9000
  API test documentation runing at: http://localhost:9000/test
==============================WELCOME================================

http://localhost:9000
  GET    /swagger.json
  GET    /swaggerui/{path*}
  GET    /swaggerui/extend.js
  GET    /test
  GET    /test/debug

start...

```
7. 编写第一个/hello 的RESTful API

在api目录里新建文件hello.js 文件名字可以根据爱好随便起。本人强烈建议使用驼峰法命名。hello.js的内容

```
module.exports = {
    method:'GET',
    path:'/api/hello',
    config:{
        description: '我的第一个hello的API'
     
    },
    handler:function(request,reply){
      
      return reply("hello light-api !");
        
    }
  
  }

```

重新启动工程
```
$ node app.js test

==============================WELCOME================================
  Welcome to LIGHT-API !!!
  Start env: test
  Server runing at: http://localhost:9000
  API test documentation runing at: http://localhost:9000/test
==============================WELCOME================================

http://localhost:9000
  GET    /api/hello                     我的第一个hello的API
  GET    /swagger.json
  GET    /swaggerui/{path*}
  GET    /swaggerui/extend.js
  GET    /test
  GET    /test/debug

start...

```
可以看到输出内容中多了一行内容 " GET    /api/hello                     我的第一个hello的API" ,这说明第一个api已经编写完成。打开浏览器（本人强烈推荐chrome，不要问为什么，这个你懂）访问http://localhost:9000/api/hello  会看到hello light-api !结果

![hello API](./image/image0.png)


8. 结束

完成以上7步，你的第一个RESTful API就开发完成了。






> ##### light_api配置说明

1. 服务的配置

配置统一配置在config的文件的目录，其中主配置放在config目录的config.js文件。config的其他目录的配置以json文件引入到config.js中。

```
module.exports = {

    getConfig:function(env){
        //引入server配置,server的配置会放在与config.j的同级目录里.如新手上路的例子中，server.json是放在/config/test目录里。
        let svr = require(`./${env}/server.json`);
        //将引入的变量配置的对应的key
        return{
            "server":svr,
            "plugins":{
               
            }
        
        }
    }
}


```

配置return对象中支持的key：

- server   必填属性 value-object
- customConfig  可选属性  value - object
- plugins  必填属性 value- object


server属性支持对server参数的配置，目前支持的属性有：
- "port":9000, 
    sever的端口号配置，必填字段, 默认端口为9000;

- "documentation":false
    server是否启用API测试文档,非必填字段 ，默认false 。若果设为true则为启用，设置为false则为不启用。建议测试环境与开发环境可设置true，生产环境可以设为false，不启用。
    当设置启用后，工程启动后。可以访问http://localhost:port/test 地址，看到服务所提供的所有API文档。

customConfig 属性支持自定义配置的引入。

```
customConfig:{key1,"value1"};

//插件中获取方法
 //引入server-utils
 let Utils = require('server-utils');
 //初始化实例
 let utils = new Utils(server);
 //获取customConfig对象中的key对应的值
 let value = utils.getConfig(key1)


//api 的handler获取的方法

handler:function(request,reply){
    //从handler的上下文this中获取
    let value = this.getConfig(key1);
    
    return reply("hello light-api !");
    
}

```

plugins 属性主要用于插件的注册

自定义的插件，插件名使用相对路径，options是插件的配置参数，如果没有参数直接设置成空对象｛｝。也可以通过外部json文件引入到config.js中使用。

```
"plugins":{插件名：options}

```


2. 启动服务时的配置加载

启动命令 “node app.js test” 的第二个参数为脚本的引用目录，启动是，server根据第二个参数，配置加载到config.js的主配置文件中，然后被server加载，成功启动server。

```
 let svr = require(`./${env}/server.json`); 
 //其中config.js的配置应用中 ${env} 就是启动命令第二个参数。

```

> ##### light_api的API开发说明

1. 开发的所用api的js文件必须放置工程的api目录里或api的子目录里，server启动时才能被自动加载。切记api目录中不能放置其他非api的js文件（非api的js文件是指不是api格式的js文件）。

2. api的js文件模版：
```
module.exports = {
    method:'GET',
    path:'/api/hello',
    config:{
        description: '我的第一个hello的API'
     
    },
    handler:function(request,reply){
      
      return reply("hello light-api !");
        
    }
  
  }

```

也支持一个js文件配置多个请求，（一般建议一个js文件对应一个api开发）
```
module.exports =[{
    method:'GET',
    path:'/api/hello1',
    config:{
        description: '我的第一个hello的API'
     
    },
    handler:function(request,reply){
      
      return reply("hello light-api !");
        
    }
  
  },{
    method:'GET',
    path:'/api/hello2',
    config:{
        description: '我的第二个hello的API'
     
    },
    handler:function(request,reply){
      
      return reply("hello light-api !");
        
    }
  
  }
]

```
其实就是一个exports的对象或对象数组，其中对象必须包含请求方法属性method,请求路路径属性path，api的配置属性config ，请求处理方法属性handler。

- method属性

http的相关请求方法,eg 'GET','POST','PUT','DELETE'。注意方法必须是大写的字符串。

- path属性

http请求的资源路径。注意必须'/'开头，否则会校验路径错误的。

- config属性

api的配置属性。可以配置权限，文档描述，请求参数的校验。

```
config:{
      
      description: '上传文件',//api描述，对应测试文档的api描述
      auth:false, //权限配置，与权限的插件配合使用
      plugins: { //文件上传时，测试文档的配置，一般api，不需要配置
            'hapi-swagger': {
                payloadType: 'form'
            }
        },
      validate: {//请求参数的验证配置
        payload: {
                file: Joi.any()
                    .meta({ swaggerType: 'file' })
                    .description('json file')
            },
        params:{
            fileType: Joi.string().required()
        }
    },
    payload:{//文件上传的大小配置，文件流的解析配置 一般api不要配置
            maxBytes:209715200,
            output:'stream',
            parse :true
    }
  },

```

- handler属性

请求处理方法属性，是api业务逻辑的处理的地方。函数的两个参数分别代表请求的request 参数与response参数。

```
handler:function(request,reply){
      
      return reply("hello light-api !");
        
}

```
请求的相关信息可以从request参数上获取。请求后的响应可以通过reply方法输出。

handler 函数中还可以通过上下文this获取相关信息，具体如下：
```
//获取插件暴漏方法及对象，参数为插件名（字符串类型）
 this.getPlugin(pluginName) ;
//获取日志对象，参数为日志所属  参数一般为__filename，使用前必须集成light-api-log插件
 this.getLogger(name);
 // 获取自定义配置中的key对应的value  参数为key（字符串类型） 配置为./config.js中的customConfig对象
 this.getConfig(key)
 //根据带文件后缀的的文件名获取mime类型。 参数为字符串
 this.getMime(filename)-

```
reply提供两个装饰方法，分别为data 与err

```
//1.data方法
reply.data("hello light-api !");
//返回带有err_no的对象 {data: "hello light-api !", err_no: 0}
//2.err方法
reply.err("请求失败");
//返回带有错误信息的对象{err_info: "请求失败", err_no: -1}，通常用于处理错误信息

```
reply还可以设置响应标准的http响应码与响应头信息header
```
//设置响应header
reply(result).header('Content-Type', 'image/png');

//响应404
reply().code(404);

```
> ##### 核心插件的使用

1.日志插件：light-api-log

- 集成

在config中注册插件
```
module.exports = {
    getConfig:function(env){
        
        let svr = require(`./${env}/server.json`); 
        let log = require(`./${env}/log.json`);
     
        return{
            "server":svr,
            "plugins":{
                "light-api-log":log
            }

        }
    }
}

```
配置log日志

/test/log.json 文件中配置日志

```
{
  "appenders": [
    { "type": "console"}//控制台输出
    {  "type": "dateFile",
       "filename": "/tmp/logs/light-data-service",
       "absolute": true,
       "alwaysIncludePattern": true,
       "pattern": "-yyyy-MM-dd.log"
     }，//日志按照特定的日期输出，本例会输出light-data-service.log-2017-09-10.log
     {
        type: 'file',
        filename: 'default.log'
    }//日志按照文件输出

  ],
  "replaceConsole": true,//增加replaceConsole配置，让所有console输出到日志中，以[INFO] console代替console默认样式。
   "level": "info" //日志输出级别
}

```

日志输出6个级别，级别分别为trace, debug, info, warn, error, fatal。

日志的appender有type有：

console ：控制台输出
file ：文件输出
dateFile：日志输出到文件，日志文件可以安特定的日期模式滚动，例如今天输出到 default-2016-08-21.log，明天输出到 default-2016-08-22.log；

- 使用
业务逻辑是写在api的js文件的hander中，因此提供日志只能在handler中使用

```
//关于日志的API demo
module.exports = {
  method:'GET',
  path:'/demo/logCase',
  config:{
      description: '日志demo'
   
  },
  handler:function(request,reply){
    //获取日志对象,标题当前目录
    let logger = this.getLogger(__filename);

    //日志级别输出控制配置/config/../log.json的level字段 
    //level 的值 all、 trace 、debug、info、warn、error、fatal 
    //eg:level:warn --此设置日志会输出warn、error、fatal 不会输出 ：trace 、debug、info
   
    //日志级别Api
    logger.trace('this is trace');
    logger.debug('this is debug');
    logger.info('this is info');
    logger.warn('this is warn');
    logger.error('this is error');
    logger.fatal('this is fatal');

    reply.data({case:"log case"});
      
  }

}

```
按照demo会输出
```
[2017-09-13 16:10:43.341] [INFO] LIGHT-API -  (log,info) data: Server runin
g at: http://localhost:9000
[2017-09-13 16:10:49.324] [TRACE] api\logger.js - this is trace
[2017-09-13 16:10:49.325] [DEBUG] api\logger.js - this is debug
[2017-09-13 16:10:49.326] [INFO] api\logger.js - this is info
[2017-09-13 16:10:49.326] [WARN] api\logger.js - this is warn
[2017-09-13 16:10:49.327] [ERROR] api\logger.js - this is error
[2017-09-13 16:10:49.328] [FATAL] api\logger.js - this is fatal
```

2.MySQL插件：light-api-mysql

- 集成

在config中注册插件

```
module.exports = {
    getConfig:function(env){
        
        let svr = require(`./${env}/server.json`); 
        let log = require(`./${env}/log.json`);
        let db = require(`./${env}/db.json`);
     
        return{
            "server":svr,
            "plugins":{
                "light-api-log":log,
                "light-api-mysql":db.mysql,
            }

        }
    }
}

```
/test/db.json 文件中配置MySQL插件
```
{
    "mysql":{  
        "connectionLimit" : 100,
        "host": "localhost",
        "user": "root",
        "password" :"1q2w3e",
        "database" :"test"
    }
   
}

```
mysql的配置属性说明：
connectionLimit：连接数的设置
host:mysql服务器的host
user:服务器的用户名
password：服务器的密码
database:使用的数据库名

- 使用

先准备好数据库。在数据库中创建一个test库然后初始化数据，初始脚本为：
```
CREATE TABLE `user` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `user_name` varchar(255) NOT NULL,
  `user_level` int(11) NOT NULL,
  `user_proj` varchar(255) DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=latin1;

-- ----------------------------
-- Records of user
-- ----------------------------
INSERT INTO `user` VALUES ('1', 'meimei', '123', 'dd');
INSERT INTO `user` VALUES ('3', 'liming', '100', 'ch');

```

```
module.exports={
    method:'GET',
    path:'/demo/mysqldemo',
    config:{
        description: 'light-api-mysql插件demo',
        validate: {
         query:{
           id:Joi.number().integer().required()
         }
        }
    },
    handler:function(request,reply){
        let rdb = this.getPlugin('light-api-mysql');
        let id = 1;
        //query方法查询
        rdb.query('SELECT * FROM user WHERE id = ?',id)
           .then(function(result){
               return reply.data(result);  
           }).catch(function(err){
               return reply.err("执行错误");
           })

    }
}

```
执行api服务 浏览器会输出:
```
{"data":[{"id":1,"user_name":"meimei","user_level":123,"user_proj":"dd"}],"err_no":0}

```
在hander中使用 let rdb = this.getPlugin('light-api-mysql');获取rdb对象。rdb提供有：增、删、改、查、分页方法，每个方法统一返回Promise对象。

基本方法:

```
 //query方法查询
    rdb.query('SELECT * FROM user WHERE id = ?',id)
        .then(function(result){
            console.log(result);
            //do something ...
        });

  //update更新方法
     rdb.update('user',id,{user_name:'newName',user_level:567})
       .then(function(result){
           console.log(result);
            //do something ...
    });

  //inset方法
    let userData = {
        user_name :zhangsan,
        user_level:9,
        user_proj:'proj'
    }

   rdb.insert("user",userData)
    .then(function(result){//result-->{id: 8112734868605768}
        console.log(result);
        //do something ...     
    });

//remove 删除方法
   rdb.remove('user',1)
    .then(function(result){
         console.log(result);
        //do something ...     
    });

//queryListPage分页查询
//返回结果包括当页数据与分页对象，分页对象-->{pageSize: 10,pageNo: 2,count: 685}
    rdb.queryListPage("select * from user ",{pageSize:10,pageNo:2})
     .then(function(result){
          console.log(result);
        //do something ...          
    });


```

事务方法:

sql的事务必须使用rdb对象的trans方法。在这个方法里通过this获取增、删、查、改、分页的方法。trans方法的sal在同一个事务中，整个trans方法执行后返回promis对象。对象的then方法中获取结果集，结果集是个数组，数组中每个元素对应trans中的单个sql的结果。

注意，tans方法的中this提供的方法不能直接获取结果集，结果统一由trans方法的then方法返回。

```
     //trans 函数包含query,update,remove,insert,queryListPage事务处理的方法。
     let userData = {
        user_name :lisi,
        user_level:10,
        user_proj:'p45'
    }
    rdb.trans(function(){
        this.query('select * from user where id=? ',id);
        this.update('user',id,{user_name:'newName',user_level:567});
        this.remove('user',id);
        this.insert("user",userData);
        this.queryListPage("select * from user",{pageSize:10,pageNo:2});
    }).then(function(result){
             console.log(result)//返回结果按照tans函数顺序放在数组中
             //do something ...
    }).catch(function(err){
            //可以捕获异常
            console.error(err);
    });

```

3.mongoDB插件：light-api-mongo

- 集成

在config中注册插件

```
module.exports = {
    getConfig:function(env){
        
        let svr = require(`./${env}/server.json`); 
        let log = require(`./${env}/log.json`);
        let db = require(`./${env}/db.json`);
     
        return{
            "server":svr,
            "plugins":{
                "light-api-log":log,
                "light-api-mysql":db.mysql,
                "light-api-mongo":db.mongoDB,
            }

        }
    }
}

```
/test/db.json 文件中配置mongoDB插件

```
{
    //这是我们上面配置的mysql的配置
 
    "mysql":{  
        "connectionLimit" : 100,
        "host": "localhost",
        "user": "root",
        "password" :"1q2w3e",
        "database" :"test"
    },
    "mongoDB":{
        "defaultDB":"portal",
        "otherDB":["admin","log"],
        "fileDB":"filedb",
        "fileColPrefix":"file",
        "poolSize":10,
        "servers":["localhost:8080"]
    }
  
}


``` 
defaultDB 默认mongodb的数据库 必填字段
otherDB：其他数据库，可以为空 非必填字段
fileDB: 文件存储数据库，非必填字段
fileColPrefix:mongoDb的GridFs的前缀，非必填字段 默认为fs
poolSize：连接池大小，非必填字段 默认10
servers: mogodb服务器host与端口 必填 [host:port] 可以是多个server

- 使用

```
module.exports={
    method:'GET',
    path:'/demo/mongodemo',
    config:{
        description: 'light-api-mongo插件demo'
    },
    handler:function(request,reply){

        //获取light-api-mysql插件服务,必须在/config/config.js中引入插件
        let sdb = this.getPlugin('light-api-mongo');

        //insert方法 插入默认db的integration集合一条文档
        sdb.insert('user',{name:'zhangsan',age:23,job:'project'})
            .then(function(result){
                return reply.data(result); //返回插入数据_id,-->5950a1c4fa7a0a2b102869d1
            }).catch(function(err){
                return reply.err('执行错误')
            });

        }
    }

```
执行api服务 浏览器会输出:
```
{
    data: "59ba35ddd326372f0ca9a302",
    err_no: 0
}

```
在hander中使用 let sdb = this.getPlugin('light-api-mongo');获取sdb对象。sdb提供有：增、删、改、查、分页方法，每个方法统一返回Promise对象。

基本方法:

注意:基本方法是默认数据库的query，insert，update，remove 方法
        /

```
    //query方法 查找默认db的集合
        sdb.query('user',{_id:'59ba35ddd326372f0ca9a302'})
        .then(function(result){
            console.log(result);
            //do something ...
            
        }).catch(function(err){
            console.error('执行错误');
        });

    //insert方法 插入默认db的integration集合一条文档
         sdb.insert('user',{name:'zhangsan',age:23,job:'project'})
         .then(function(result){
             console.log(result);//返回插入数据_id,-->59ba35ddd326372f0ca9a302
            //do something ... 
            }).catch(function(err){
              console.error('执行错误');
            });

    //update方法 更新默认db的integration集合的文档

        sdb.update('user','59ba35ddd326372f0ca9a302',{name:'lisi'})
          .then(function(result){
            console.log(result);
            //do something ... 
           }).catch(function(err){
              console.error(err);
           });

    //remove方法 删除指定文档
     sdb.remove('user',"59ba35ddd326372f0ca9a302")
        .then(function(result){
           console.log(result);
            //do something ... 
        }).catch(function(err){
            console.error(err);
           
        })


    //分页查询
        /**
            * 分页查询
            * @param {*} collectionName ：集合名 eg "integration"
            * @param {*} where ：查询条件 {}
            * @param {*} sort ：排序条件 {create_date:-1}
            * @param {*} pageParam ：分页参数 eg {pageNo:1,pageSize:10} 
            */

    sdb.queryListPage('user',{},{create_date:-1},{pageNo:1,pageSize:10})
         .then(function(result){
            console.log(result);
            //do something ...
         }).catch(function(err){  
            console.error(err); 
            
        });

    //获取文档条数
    /**
        * 查询总数
        * @param {*} collectionName ：集合名
        * @param {*} where ：查询条件 
        */

    sdb.getCount('user',{})
    .then(function(result){
        console.log(result);
        //do something ...
    }).catch(function(err){
            console.error(err); 
    });

    // 查询指定条数的数据
    /** 
        * 查询指定条数的数据
        * @param {*} collectionName ：集合名 eg 'integration'
        * @param {*} where ：查询条件 eg {}
        * @param {*} sort ：排序条件{field:-1} -1降序 1升序
        * @param {*} limit ：限制参数{skip:2,limit: 10}
        * 
        */

    sdb.queryByLimtCount('user',{},{create_date:-1},{limit:2})
      .then(function(result){
         console.log(result);
         //do something ...
        }).catch(function(err){
            console.error(err); 
        });

    //根据查询条件，删除多个文档
    /**
        * 根据查询条件，删除多个文档
        * @param {*} collectionName ：集合名 
        * @param {*} where ：删除的查询条件
        * 
        */

    sdb.removeAll('user',{"age" : "23"})
        .then(function(result){
         console.log(result);
         //do something ...
        }).catch(function(err){
             console.error(err); 
        });

```
指定非默认数据库的query，insert，update，remove 方法
配置中必须配置 "otherDB":["db1","db2"],下面demo已查admin数据库为例子

```

    // insert方法
        sdb.insert('user1',{name:'zhangsan',age:23,job:'project'},"")
           .then(function(result){
                console.log(result);
                //do something ...
             }).catch(function(err){    
                 console.error(err); 
             });

    // query方法
         sdb.query('user1',{_id:"59ba35ddd326372f0ca9a302"},"admin")
            .then(function(result){
                console.log(result);
                //do something ...
            }).catch(function(err){
                 console.error(err); 
            })

    //update方法
         sdb.update("user1",'59ba35ddd326372f0ca9a302',{name:'fangwu'},"admin")
            .then(function(result){
               console.log(result);
                //do something ...
            }).catch(function(err){
              console.error(err);
            })
            
    //remove 方法
         sdb.remove('user1','59ba35ddd326372f0ca9a302',"admin")
           .then(function(result){
                console.log(result);
                //do something ...
            }).catch(function(err){
               console.error(err);
            })

    //分页查询
        /**
            * 分页查询
            * @param {*} collectionName ：集合名 eg "board"
            * @param {*} where ：查询条件 {}
            * @param {*} sort ：排序条件 {create_date:-1}
            * @param {*} pageParam ：分页参数 eg {pageNo:1,pageSize:10}
            * @param {*} dbName ：monodb的数据库名 eg "admin"
            * 
            */
        sdb.queryListPage('user1',{},{create_date:-1},{pageNo:1,pageSize:10},"admin")
          .then(function(result){
              console.log(result);
              //do something ...
         }).catch(function(err){
             console.error(err);
         });
    
        
    //获取文档条数
        /**
            * 查询总数
            * @param {*} collectionName ：集合名
            * @param {*} where ：查询条件
            * @param {*} dbName ：monodb的数据库名
            * 
            */
        sdb.getCount('user1',{},'admin')
             .then(function(result){
               console.log(result);
               //do something ...
             }).catch(function(err){
                console.error(err);
             });

    // 查询指定条数的数据
        /**
            *  查询指定条数的数据
            * @param {*} collectionName ：集合名 eg 'board'
            * @param {*} where ：查询条件 eg {}
            * @param {*} sort ：排序条件{field:-1} -1降序 1升序
            * @param {*} limit ：限制参数{skip:2,limit: 10}
            * @param {*} dbName ：monodb的数据库名 eg 'live'
            * 
            */

         sdb.queryByLimtCount('user1',{},{create_date:-1},{limit:2},'admin')
             .then(function(result){
                 console.log(result);
                 //do something ...
             }).catch(function(err){
                 console.error(err);
            });
    //根据查询条件，删除多个文档
        /**
            * 根据查询条件，删除多个文档
            * @param {*} collectionName ：集合名
            * @param {*} where ：删除的查询条件
            * @param {*} dbName ：monodb的数据库名
            * 
            */

        sdb.removeAll('user1',{'name' : 'wangwu'},'admin')
            .then(function(result){
                 return reply.data(result);
           }).catch(function(err){
                 return reply.err('执行错误');
          });

```

4.redis插件：light-api-redis

-- 在config中注册redis插件

```
module.exports = {
    getConfig:function(env){
        
        let svr = require(`./${env}/server.json`); 
        let log = require(`./${env}/log.json`);
        let db = require(`./${env}/db.json`);
     
        return{
            "server":svr,
            "plugins":{
                "light-api-log":log,
                "light-api-mysql":db.mysql,
                "light-api-mongo":db.mongoDB,
                "light-api-redis":db.redis
            }

        }
    }
}

```
/test/db.json 文件中配置redis插件

```
{
    //这是我们上面配置的mysql的配置
 
    "mysql":{  //之前mysql的配置
        "connectionLimit" : 100,
        "host": "localhost",
        "user": "root",
        "password" :"1q2w3e",
        "database" :"test"
    },
    "mongoDB":{//之前redis的配置
        "defaultDB":"portal",
        "otherDB":["admin","log"],
        "fileDB":"filedb",
        "fileColPrefix":"file",
        "poolSize":10,
        "servers":["localhost:8080"]
    },
    "redis":{
        "sentinels":["localhost:63791"],
        "name": "mymaster",
        "password":"123123",
        "cache":true,
        "mq":true,
        "channels":["channel1"]
    }
  
}


``` 
sentinels redis主从模式的的host与port 必填字段
name：sentinels 监控主redis的名字，可选字段 默认为mymaster
fileDB: 文件存储数据库，可选字段
password:redis的密码 可选字段 默认不设密码
cache：开启redis的缓存功能，可选字段 默认开启
mq：开启消息订阅功能 可选字段 默认关闭
channels：配置消息订阅的渠道 可选字段与mq设置有关，mq开启时才有效


-- 使用

```
module.exports={
    method:'GET',
    path:'/demo/redisdemo',
    config:{
        description: 'light-api-redis插件demo'
    },
    handler:function(request,reply){
        let cache = this.getPlugin('light-api-redis').cache;
        cache.set('demokey',{name:'demo'})
             .then(function(result){
                return reply.data(result)  
            }).catch(function(err){
                 return reply.err('执行错误');
            });
    }
}

```
执行API ，浏览器访问uri /demo/redisdemo ，返回结果：
```
{
    data: "OK",
    err_no: 0
}

```
redis缓存的使用:在hander中使用 let red = this.getPlugin('light-api-redis');获取red对象。 在由red对象获取cache服务，let cache = red.cache,cache提供有：增、删、改、查、分页方法，每个方法统一返回Promise对象。

注意：使用cache功能需要确定/config/../db.json中的cache:true,否则就是不用配置，使用默认配置

```
//set方法 存入缓存 key-->value  value可以是字符串也可以是对象
     cache.set('key1',{name:'demo'})
         .then(function(result){
             console.log(result);
             //do something...  //结果返回--->'ok'
         }).catch(function(err){
             console.error(err);
         });

//get方法 更具key获取缓存数据
    cache.get('key1')
        .then(function(result){
              console.log(result);
              //do something...
        }).catch(function(err){
            console.error(err);
        });

//remove方法 更具key获取缓存数据
     cache.remove('key1')
         .then(function(result){
            console.log(result) //结果返回---->1
            //do something ... 
          }).catch(function(err){
             console.error(err);
          });

    //setTtl方法 设置存入cache的数据的时效时间 单位未秒
      cache.setTtl('key1',{name:'demo'},60)
          .then(function(result){
              console.log(result);// 结果返回--->'ok'
              //do something ...
              return reply.data(result) 
           }).catch(function(err){
              console.error(err);
           });

     //getExpireTime方法 获取key的过期时间 单位为秒
        cache.getExpireTime('key1')
             .then(function(result){
                 console.log(result);//返回结果--->时效时间 单位为秒
                 //do something ...;
              }).catch(function(err){
                  console.error(err);
              });


```

redis消息订阅使用：在hander中使用 let red = this.getPlugin('light-api-redis');获取red对象。 在由red对象获取mg服务，let mq = red.mq,mq提供消息的发送方法与消息接收方法.

注意:/config/../db.json开启mq配置为true,channels配置中增加消息渠道

```   
      let mq = this.getPlugin('light-api-redis').mq;     
      
      向渠道'channel1'发送消息
      mq.sender('channel1',{name:'nb',age:23});

      向渠道'chanel1接收消息（此功能基本用于插件）
       mq.onMessage(['channel1'],function(err,message){
            console.log(message);
       });)
```

> ##### 自定义插件的开发

插件在server启动后被加载，属于中间件。使用插件可以开发server的辅助功能。比如权限控制。

插件的模版代码:
```
exports.register = function (server, options, next) {
    //do semothing ...
    next();
};

exports.register.attributes = {
    name: 'my-plugin-demo',
    version: '1.0.0'
};


```
register函数的参数：
server ：整个light-api的server对象
options：配置对象 对应在config插件配置的，插件名对用的value。可以用来获取插件的预配置信息。
next：整个插件业务处理完后一定要执行next(),负责整个sever就会阻塞在此处。

register.attributes：
 
插件的属性，会标明插件的名称与插件的版本。

注意 在config.js中配置的时候，自定义插件义路径配置

-- 开发过程

规范：所有的自定义插件都放在plugin目录中

1步：在plugin目录中新建pluginDemo.js

2步：开发代码
```

let Utils = require('server-utils');

var service = {
    say:function(){
        return 'hellow LIGHTAPI'
    }
}

/**
 * @param {*} server ：light-api server对象
 * @param {*} options ： ./config/config.js 注册插件的配置 
 * @param {*} next ：最后必须调用next()
 */
exports.register = function (server, options, next) {
    //通过server-utils 工具可以获取其他插件暴漏的方法
    let utils = new Utils(server);
    let rdb = utils.getPlugin('light-api-mysql');

    //插件暴漏方法必修写在service上，在API的handler中可以使用this.getPlugin('pluginName')获取对象
    server.expose('service', service);
    
    
    //插件的日志输出使用server.log('info',message);
    server.log('info','completed to my-plugin-demo plugin init');

   //重要提醒：一定不要忘记调此方法（重要的事情说三遍）。
    next();
};

exports.register.attributes = {
    name: 'my-plugin-demo',
    version: '1.0.0'
};




```
在自定义插件开发时，可以引用server-utils 模块 ，通过此模块可以获取其他插件暴漏的方法与对象

 * server-utils 模块暴漏的属性方法如一下：
 *  1.getPlugin(pluginName) --获取插件暴漏方法及对象，参数为插件名（字符串类型）
 *  2.getLogger(name) -- 获取日志对象，参数为日志所属  参数一般为__filename
 *  3.getConfig(key) -- 获取自定义配置中的key对应的value  参数为key（字符串类型） 配置为./config.js中的customConfig对象
 *  4.getMime(filename)--根据带文件后缀的的文件名获取mime类型。 参数为字符串
 

3步: config.js中配置自定插件
```
module.exports = {
    getConfig:function(env){
        
        let svr = require(`./${env}/server.json`);
        let db = require(`./${env}/db.json`);
        let log = require(`./${env}/log.json`);
       
        return{
            "server":svr,
            "plugins":{
                "light-api-log":log,
                "light-api-mysql":db.mysql,
                "light-api-mongo":db.mongoDB,
                "light-api-redis":db.redis,
                "./plugin/pluginDemo.js":{}
            }

        }
    }
}

```

4步：在api中使用我们开发的插件

在handler中可以通过插件名调用插件暴漏的服务对象，调用let myPlugin = this.getPlugin(插件名)

```
module.exports={
    method:'GET',
    path:'/demo/pluginDemo',
    config:{
        description: 'pluginDemo插件api'
    },
    handler:function(request,reply){
        //使用插件名获取插件暴漏服务对象
        let myPlugin = this.getPlugin('my-plugin-demo');
        
        return reply.data(myPlugin.say());
    }
}

```

5步：在浏览器中调用/demo/pluginDemo 的api，输出结果：
```
{
   data: "hellow LIGHTAPI",
   err_no: 0
}
```

以上为自定插件的开发，使用。