let config = require("../../kernel/config");
let utils = require("../../kernel/util");
let cheerio = require("cheerio");

/**
 * 将html文件合并为可以运行的代码片段
 */
module.exports = function (options) {
    return new Promise(function (resolve) {
        utils.glob(`${config.dist}/*.html`,function (err, files) {
            resolve(files);
        });
    }).then(function (files) {
        let filePromise = [];

        files.forEach(function (file) {
            filePromise.push(new Promise(function (resolve) {
                //1. 基本处理
                let content = utils.readFileSync(file).toString().replace(/\r\n/ig, "\n");//处理换行符

                //处理为$元素
                let $ = loadAsJQuery(content);

                resolve($);
            }).then(function ($) {
                //2. 合并视图/组件/snippet
                let scripts = [];

                utils.log2("info","assemble.html",utils.parse(file).name);
                ["view","component","snippet"].forEach(function (tagName,index,tags) {
                    asmblePices($,tagName,scripts);
                });

                //3. 注入JS文件
                $("sub-view").attr("sub-view","").each(function (i,dom) {
                    dom.tagName = "div";
                });
                // let content = $.html().replace(/sub\-view/ig, "div");
                let content = $.html();

                utils.log2("info","assemble.js",utils.parse(file).name);

                let spiltedContent = content.split(/<!--\s*inject:view\s*-->\s*<!--\s*endinject\s*-->/ig);

                if (spiltedContent.length === 2) {
                    let scriptString = [`<script type='text/javascript' src='js/regist/${utils.parse(file).name}.js'></script>`];
                    scripts.reverse().forEach(function (script) {
                        scriptString.push(`<script type='text/javascript' src='${script}'></script>`);
                    });

                    content = spiltedContent[0] + scriptString.join("\n") + spiltedContent[1];
                }

                return content;
            }).then(function (content) {
                //END 资源处理完成，写入原来的文件
                utils.writeFileSync(file, content);
            }))
        });

        return Promise.all(filePromise);
    })
};

/**
 * 代码片段，view，comp，snippet合并,并且注入js到inject注解区域
 * @param $
 * @param tagName
 * @param scripts 脚本
 */
function asmblePices($, tagName,scripts) {
    let fragments = $(tagName);

    let parent_child_map = {};//如果是视图的话存在父子嵌套视图
    fragments.each(function (i, fragment) {
        let attr = fragment.attribs;


        //1. 资源整合：处理html片段
        let html = `${config.dist}/html/${tagName}/${attr.id}.html`,
            js = `js/${tagName}/${attr.id}.js`;

        //2. 资源整合:工程间资源依赖处理,视图的html资源
        if (attr.ref) {
            let ref = /^@([^\/]*)(.*)$/.exec(attr.ref);
            if (ref && ref[1] && config.config && config.config.dependency && config.config.dependency[ref[1]]) {
                let depPath = config.config.dependency[ref[1]].path || '';

                //处理资源依赖:html
                html = `${depPath}/src/html/${tagName}${ref[2]}.html`;

                //处理资源依赖:js
                utils.copySync(`${depPath}/src/js/${tagName}${ref[2]}.js`,`${config.dist}/src/js/${tagName}${ref[2]}.js`);
            } else {
                utils.log2("info","dependency.notfound",ref[1])
            }
        }

        if(!attr.async || attr.async == "false") {
            //如果不是异步，则还是按照原来的逻辑处理
            //3. 资源整合：合并html资源
            //父子关系还是需要在编译期处理
            parent_child_map[attr.id] = attr;

            if (utils.existsSync(html)) {
                $(fragment).replaceWith(utils.readFileSync(html).toString());
            }

            //4. 资源整合：处理js资源要区分是否为异步视图
            if (utils.existsSync(`${config.dist}/${js}`)) {
                scripts.push(js);
            }
        } else {
            if (attr.async === "mixed") {
                attr.id = utils.camel(attr.id);
                let viewContent = "";
                if (utils.existsSync(html)) {
                    viewContent = `/**START*/Light["viewStrings-#${utils.camel(attr.id)}"] = "${encodeURIComponent(utils.readFileSync(html).toString())}";/**END*/`;
                }

                //DIST目录里这个文件在开发阶段如果重复变更的话会出现问题，重复出现html文件里dom的内容
                if (utils.existsSync(`${config.dist}/${js}`)) {
                    let jsContent = utils.readFileSync(`${config.dist}/${js}`).toString(),
                        mixedCheck = /^\/\*\*START\*\/(.*)\/\*\*END\*\//ig.exec(jsContent);
                    if (mixedCheck) {
                        jsContent.replace(mixedCheck[0], viewContent);
                    } else {
                        viewContent = `${viewContent};${jsContent}`
                    }
                }
                utils.writeFileSync(`${config.dist}/${js}`, viewContent);
            } else {
                //兼容原有模式
                parent_child_map[attr.id] = attr;

                if (utils.existsSync(html)) {
                    $(fragment).replaceWith(utils.readFileSync(html).toString());
                }
            }
        }


        //5. 资源整合：如果是视图的节点还要处理父子节点关系
        //无论如何都应该兼容原有的东西
        //如果要将js和html进行合并处理，那么组装不能不能发生编译期，组装只能发生在运行期
        Object.keys(parent_child_map).forEach(function (key) {
            if(parent_child_map[key].parent){
                // 当一个视图内包含多个sub-view的dom节点时,只有最靠近前面的一个生效
                // 因为，当存在多级视图时，无法准确的知道组装顺序
                $(`#${utils.camel(parent_child_map[key].parent)}`).find("sub-view").first().append($(`#${utils.camel(key)}`))
            }
        });
    });
}

/**
 * 将内容中的内容处理为jquery对象方便选择器工作
 * @param content
 * @returns {*}
 */
function loadAsJQuery(content) {
    return cheerio.load(content, {
        recognizeSelfClosing: true,
        decodeEntities: false
    });
}