let utils = require("../../kernel/util");
let config = require("../../kernel/config");

let server = require('http').createServer(),
    WebSocketServer = require('ws').Server,
    express = require("express"),
    url = require('url'),
    wss = new WebSocketServer({server: server});

wss.setMaxListeners(0);

/**
 * 打开浏览器并开启server
 */
module.exports = function (options) {

    if (!options.browser){
        return Promise.resolve();
    }

    require("../build/plugin").runStage("serve").then(function (){
        let app = express();
        let port = options.browser === true ? 3000 : options.browser;

        let sockets = [];
        wss.on('connection', function (socket) {
            if (socket.id) return;

            socket.id = sockets.push(socket);
            socket.setMaxListeners(0);
            socket.on('message', function (message) {
                message = JSON.parse(message);
                if (message.type === "log") {
                    utils.log2(message.level,"msg.from.local", message.userAgent, message.message);
                }
            });

            socket.on('close', function () {
                delete sockets[socket.id];
            });
        });


        app.get(["**/*\.html", "/", "/_index"], function (req, res) {
            let path = req.path;
            if (path === "/") {
                path = "/index.html";
            }

            //支持JSN,动态程序入口
            if(path === "/_index"){
                const userAgent = req.headers['user-agent'];
                if(userAgent.indexOf("LightOS/JSN")>-1){
                    path = "/app.native.js";
                }else{
                    path = "/index.html";
                }
            }


            res.set('Content-Type', 'text/html');
            utils.readFile(utils.join(config.dist, path),function (err, html) {
                html += `
                    <script>
                        var ws = new WebSocket('ws://'+location.host);
                        if(window.Light) window.Light.Logger.websocket = ws;
                        ws.onmessage = function(data, flags) {
                          ws.close();
                          location.reload();
                        }
                    </script>
                `.replace(/\n/ig, "");

                res.send(new Buffer(html));
                res.end();
            });
        });

        //middleware support
        if(global.middleware){
            for(let mid of global.middleware){
                app.use(mid)
            }
        }

        app.use(express.static(config.dist));

        server.on('request', app);
        server.listen(port);


        if(!options.stayStill){
            let exec = require('child_process').exec,
                cmd;
            switch (process.platform) {
                case 'win32':
                    cmd = 'start';
                    break;

                case 'linux':
                    cmd = 'xdg-open';
                    break;

                case 'darwin':
                    cmd = 'open';
                    break;
            }
            // exec(cmd + " http://localhost:" + port);
            exec(cmd + " http://" + utils.getIPAdress() + ":" + port);
            
        }

        

        return Promise.resolve(sockets);
    })
};