let utils = require("../../kernel/util");
let config = require("../../kernel/config");
let cheerio = require('cheerio');
let zipDir = utils.zipDir;

module.exports = function (options) {
    return new Promise(function (resolve) {
        utils.glob(`${config.dist}/*.html`,function (err, files) {
            resolve(files);
        });
    }).then(function (files) {
        let filePromise = [];

        files.forEach(function (file) {
            filePromise.push(new Promise(function (resolve) {
                //1. 基本处理
                let content = utils.readFileSync(file).toString().replace(/\r\n/ig, "\n");
                resolve(content);
            }).then(function (content) {
                //5. 资源统一处理：合并
                if(options.concat){
                    return require("./concat")(content);
                }else{
                    return Promise.resolve(content);
                }
            }).then(function (content) {
                //5. 资源统一处理：压缩
                if(options.uglify){
                    return require("./uglify")(cheerio.load(content, {
                        recognizeSelfClosing: true,
                        decodeEntities: false
                    }));
                }else{
                    return Promise.resolve(content);
                }
            }).then(function (content) {
                //5. 资源统一处理：后缀
                if(options.suffix){
                    return require("./suffix")(cheerio.load(content, {
                        recognizeSelfClosing: true,
                        decodeEntities: false
                    }));
                }else{
                    return Promise.resolve(content);
                }
            }).then(function (content) {
                //END 资源处理完成，写入原来的文件
                utils.writeFileSync(file, content);
            }))
        });

        return Promise.all(filePromise).then(function () {
            return require("./concat").processConcatMap();
        });
    }).then(function () {

        /**
         * 插件：packager
         */
        if (options.pack) {
            return require("./plugin").runStage("packager");
        }
    }).then(function () {
        if (options.pack) {

            let getZipFile = function (env) {
                if(options.packageName){
                    return `${options.packageName}.zip`;
                }
                return `${config.name || config.project}-${config.version}${env ? '-' + env : ''}_${new Date().Format("yyyyMMdd")}.zip`
            };

            let env, files = [];
            if (options.pack !== true) {
                env = [options.pack];
            } else if (config.config && config.config.packager) {
                env = utils.clone(config.config.packager.env) || [];
            } else {
                env = [];
            }

            if (config.config && config.config.packager) {
                files = config.config.packager.files || [];
            }

            if (env.length === 0 || files.length === 0) {
                utils.log2("info", "packager.start","");
                return zipDir(config.dist, `${config.src}/../${getZipFile()}`,config);
            } else {
                let readQueue = function () {
                    let e = env.pop();

                    if (!e) return;

                    utils.log2("info", "packager.start",e);
                    files.forEach(function (f) {
                        let origin = f.replace(/\$\{env\}/ig, e),
                            to = f.replace(/[\._]?\$\{env\}/ig, "");
                        utils.copySync(
                            utils.join(config.dist, origin),
                            utils.join(config.dist, to)
                        )
                    });
                    return zipDir(config.dist, `${config.src}/../${getZipFile(e)}`).then(function () {
                        return readQueue();
                    },config);
                };

                return readQueue();
            }
        }else{
            return Promise.resolve();
        }
    })
};