
let config = require("../../kernel/config");
let utils = require("../../kernel/util");
let pluginsRuntime = [];

/**
 * 加载并初始化插件
 */
module.exports = function (options) {
    let plugins = [];

    plugins = config.plugins || [];

    //如果是olight版本，默认直接开始olight插件
    if(config.type==="vue"){
        plugins.unshift("type-vue");
    }

    let cliPlugin = options.enablePlugins !== true ? options.enablePlugins:null;

    if(cliPlugin&&plugins.indexOf(cliPlugin)<0){
        plugins.push(cliPlugin)
    }

    //1. 判断插件目录是否存在
    if(!utils.existsSync(config.pluginsDir)){
        utils.mkdirpSync(config.pluginsDir)
    }

    //2. 检查是否安装过此插件
    let checkPluginPromise = [];

    plugins.forEach(function (plugin) {
        checkPluginPromise.push(new Promise(function (resolve) {
            let pluginDir = utils.join(config.pluginsDir, `node_modules/lighting-plugin-${plugin}`);

            if (utils.existsSync(pluginDir)) {
                resolve(pluginDir)
            } else {
                //3. 插件不存在，则安装

                utils.log2("info","plugin.notfound",plugin);
                utils.log2("info","plugin.install",plugin);

                require("../plugin").do(null, {
                    add: plugin
                }, function () {
                    utils.log2("info","plugin.install.success",plugin);
                    resolve(pluginDir)
                });
            }
        }).then(function (pluginDir) {
            let plugin = require(pluginDir);
            if (plugin.install) plugin.install(light);//安装插件成功
            pluginsRuntime.push(plugin);
        }))
    });

    return Promise.all(checkPluginPromise);
};

/**
 * 运行某一阶段的插件
 * @param stage
 * @param fileEXT
 */
module.exports.runStage = function (stage,fileEXT) {
    let pluginsRunner = [];
    pluginsRuntime.forEach(function (plugin) {
        pluginsRunner.push(new Promise(function (resolve) {
            if (plugin[stage] && typeof plugin[stage] === "function") {
                if(fileEXT && plugin.buildType && plugin.buildType.indexOf(fileEXT)<0){
                    return resolve();
                }

                utils.log2("info","plugin.invoke",stage,plugin.pluginName);
                plugin[stage].call(plugin, light, function () {
                    utils.log2("info","plugin.invoke.success",stage,plugin.pluginName);
                    resolve();
                });
            } else {
                resolve()
            }
        }))
    });

    return Promise.all(pluginsRunner);
};