const {log,openURL} = require("../../kernel/util");

module.exports = function (pkg,options) {
    log("info",`正在准备提交发布包...`);

    let appId = new Promise(function (resolve,reject) {
        //1. 获取组件id
        let app_id = require("../../kernel/config").link_id;

        if(!app_id){
            reject(`当前工程还未关联应用ID，请在package.json或project.json中配置link_id属性！`);
        }else{
            resolve(app_id);
        }
    });

    let loginInfo = new Promise(function (resolve,reject) {
        const authStore = require("../../kernel/config").authStore;

        require("fs").readFile(authStore,function (err, data) {
            if(err){
                return reject(`用户还未登录，请使用${'light login'.green}登录后继续操作！`)
            }
            resolve(JSON.parse(data))
        })
    });

    let promise = loginInfo.then(function () {
        return new Promise(function (resolve,reject) {
            require("request").post({
                url:'https://light.hscloud.cn/data/file/light/package',
                formData: {
                    file: require("fs").createReadStream(pkg),
                }
            }, function(err, httpResponse, body) {
                if (err) {
                    reject('upload failed:', err);
                }
                resolve(JSON.parse(body).data)
            });
        })
    });

    //已经获得了上传包，下面进行版本的发布操作，发布之前必须绑定过app——id
    promise = Promise.all([loginInfo,promise,appId]).then(function (checkData) {
        let file = checkData[1];
        let login = checkData[0];
        let app_id = checkData[2];

        return new Promise(function (resolve,reject) {
            require("request").post({
                url:options.path,
                form: options.getParams({app_id,file}),
                headers:{
                    Authorization:`UserToken ${login.long_token}`,
                    LightAuth:encodeURIComponent(JSON.stringify({
                        auth_id:login.auth_id,
                        user_token:login.long_token,
                        device_uid:login.deviceUid,
                    }))
                }
            }, function(err, httpResponse, body) {
                if (err) {
                    reject('网路错误', err);
                }
                resolve(JSON.parse(body).data)
            });
        })
    });

    //版本创建成功
    promise = promise.then(function (data) {
        log("info","版本创建成功，可以扫描下方的二维码体验！");

        // 59982		【lighting】“light release --upload”: 兼容性
        openURL(`https://light.hscloud.cn/data/utils/qrcode?url=`+encodeURIComponent(options.getUrl(data)))
        // require('qrcode-terminal-alpha').generate(options.getUrl(data),{small:true})
    });

    promise.catch(function (err) {
        log("error",err);
    })
};