let utils = require("../../kernel/util");
let config = require("../../kernel/config");
let chokidar = require("chokidar");

let watcher = null;

/**
 * watch指令的使用禁忌：
 * 1. 不可与打包指令同时使用
 * 2. 不可与打包阶段插件同时使用
 * @param options
 */
module.exports = function (options,server) {
    server = server || [];

    //WATCH：开启处理
    if (options.watch) {

        utils.log2("info","watch.start");

        let watchTimeout;
        watcher = chokidar.watch(config.src, {ignored: /(.*node_modules\/.*|.*.idea\/.*)/}).on('change', function (file) {
            if (watchTimeout) {
                clearTimeout(watchTimeout);
                watchTimeout = null;
            }

            watchTimeout = setTimeout(function () {
                utils.log2("info","watch.start.build",file);

                //有变化则理解拷贝变化
                utils.removeSync(file.replace(config.src,config.dist));//先删除此文件
                utils.copySync(file,file.replace(config.src,config.dist));

                /**
                 * VUE框架下的不需要重现编译直接使用
                 */
                if(config.type==="vue"&&file!==`${config.src}/index.html`){
                    utils.log2("info","lighting.end");
                    // server.forEach(function (socket) {
                    //     if (socket && socket.readyState === 1) socket.send('reload');
                    // });
                    return;
                }

                // 59500		【lighting】增加样式文件的增量编译能力
                if(file.endsWith(".less")||file.endsWith(".css")){
                    return require("./plugin").runStage("prepare","less");
                }

                /**
                 * 资源的增量编译：
                 * 1. 增量编译只对插件的调用和处理有效，必须关联特殊后缀的资源（非js/css）
                 * 2. 重演一遍build指令的路径吗？
                 * 3. 需要抽象成单独的文件吗或者单独的代码片段
                 */

                let stepPromise = Promise.resolve();
                let ext = utils.parse(file).ext.substring(1);

                stepPromise = stepPromise.then(function () {
                    return new Promise(function (resolve) {
                        utils.glob(`${config.src}/{*.html,html/page/*.html,html/page/*.json}`,function (err,files) {
                            let moveFiles = [];

                            files.forEach(function (file) {
                                moveFiles.push(new Promise(function (resolve) {
                                    if(file.endsWith("json")){
                                        let page = require(`${process.cwd()}/${file}`);
                                        //
                                        page.name = utils.parse(file).name;
                                        //
                                        // //先判断工程目录里有没有模板文件
                                        let pageTemplate = `${config.src}/html/page/page.template`;
                                        if(!utils.existsSync(pageTemplate)){
                                            pageTemplate = `${__dirname}/../../scaffold/page.template`;
                                        }
                                        //
                                        let pageContent = require("./template").template(pageTemplate,{
                                            page:page,
                                            project:config
                                        });
                                        //
                                        utils.writeFile(`${config.dist}/${utils.parse(file).name}.html`,pageContent,function () {
                                            resolve()
                                        })
                                    }else{
                                        let dist = utils.join(config.dist, utils.parse(file).base); //搬迁所有的html/page下的文件到根目录
                                        // utils.copy(file, dist,function () {
                                        //     resolve()
                                        // });
                                        //这样竟然会时序不对，到时候要研究一下
                                        // utils.copySync(file, dist);
                                        // 姜：多次修改页面空白的问题
                                        utils.writeFileSync(dist,utils.readFileSync(file));
                                        resolve();
                                    }
                                }))
                            });

                            return Promise.all(moveFiles).then(function () {
                                resolve();
                            });
                        });
                    })
                });

                /**
                 * 插件：prepare
                 */
                stepPromise = stepPromise.then(function () {
                    return require("./plugin").runStage("prepare",ext);
                });

                /**
                 * 编译
                 */
                stepPromise = stepPromise.then(function () {
                    return require("./assemble")(options)
                });

                /**
                 * 插件：build
                 */
                stepPromise = stepPromise.then(function () {
                    return require("./plugin").runStage("build",ext);
                });

                /**
                 * 后处理
                 */
                stepPromise.then(function () {
                    utils.log2("info","lighting.end");
                    server.forEach(function (socket) {
                        if (socket && socket.readyState === 1) socket.send('reload');
                    })
                });
            }, 500);
        });
    }

    return Promise.resolve();
};