//build引入webpack让开发者自己处理依赖和数据关系
//如果开发者自己管理的话那么需要接触那些资源，入口的js是什么，
//配置文件是什么

exports.do = function(cmd,options) {
    if (!require("fs").existsSync("package.json") && !require("fs").existsSync("project.json")) {
        console.log("请在Lighting工程目录执行release操作！");
        return;
    }

    /**
     * 第0步，准备
     */
    if(options.watch){
        process.env.NODE_ENV = "dev";
    }

    if (options.product) {
        options.suffix = true;
        options.uglify = true;
        options.concat = true;
    }

    if (options.upload) {
        options.pack = true;
    }

    light.options = options;

    let stepPromise;

    /**
     * 第一步，环境初始化
     */
    stepPromise = require("./build/reset")(options);

    /**
     * 第二步，插件加载
     */

    stepPromise = stepPromise.then(function () {
        return require("./build/plugin")(options)
    });

    /**
     * 插件：prepare
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/plugin").runStage("prepare");
    });

    /**
     * 第三步，page页面的资源装配，这里就要分页面单独处理了
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/assemble")(options)
    });

    /**
     * 插件：build
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/plugin").runStage("build");
    });

    /**
     * 第四步，资源打包输出zip包
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/packager")(options)
    });

    stepPromise = stepPromise.then(function (pkg) {
        if(options.upload && pkg){
            let params = {
                path:"https://light.hscloud.cn/data/service/portal/app/create_version",
                getParams(data){
                    return {
                        app_id:data.app_id,
                        package:data.file,
                        is_flow_install:1,
                        remark:`uploaded by lighting(${new Date().Format('yyyy-MM-dd hh:mm:ss')})`,
                        version: require("../kernel/config").version
                    }
                },
                getUrl(data){
                    return `http://light.hscloud.cn/apps/${data.version.name}-test/index.html?_=${new Date().getTime()}`;
                }
            };

            //如果使用了native插件就提交native包
            if(light.config.plugins&&light.config.plugins.indexOf("native")>-1){
                params = (function () {
                    let packData = null;
                    return {
                        path: "https://light.hscloud.cn/data/service/portal/appdiy/setAppGmuFile",
                        getParams(data) {
                            packData = data;
                            return {
                                gmu_file_id: data.file,
                                //51342 【优化】Lighting打包后，版本包上传到light的名称统一为APP-版本号（时间戳）
                                upload_desc: `${light.config.project}-V${light.config.version}(${new Date().Format('yyyyMMdd hh:mm:ss')})`,
                                native_app_id: data.app_id,
                            }
                        },
                        getUrl(data) {
                            return `https://app.lightyy.com/appDist/ShowSoloApp?appName=com.hundsun.light.lightIn.appstore&viewinfo=${encodeURIComponent(JSON.stringify({
                                share_app: packData.app_id,
                                share_app_file: packData.file,
                                create_time:new Date().getTime()
                            }))}`;
                        }
                    }
                })()
            }

            //将发布包提交到平台
            return require("./build/upload")(pkg,params)
        }
    });

    /**
     * 第五步，-b选项的处理，开启内部server
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/browser")(options)
    });

    /**
     * 第六步，watch参数的处理，关系到资源的增量编译
     */
    stepPromise = stepPromise.then(function (server) {
        return require("./build/watch")(options,server)
    });

    /**
     * THE END
     * 结束资源编译
     */
    stepPromise = stepPromise.then(function () {
        return require("./build/end")()
    })

    return stepPromise;
};